unit BtnEdit;

{
  TBtnEdit
  Copyright (C) 1998-2001 Campulka Jiri
  E-mail: campulka@seznam.cz
          jiri@campulka.net
  WWW   : http://campulka.net
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TBtnEdit = class(TEdit)
  private
    { Private declarations }
    fButtonWidth: Integer;
    fPressed: Boolean;
    fInButton: Boolean;
    fOnButtonClick: TNotifyEvent;
    fBtnCaption: String;
    procedure SetButtonWidth(Value: Integer);
    procedure SetBtnCaption(Value: String);
    procedure WMLButtonDown(var M: TWMMouse); message wm_LButtonDown;
    procedure WMLButtonUp(var M: TWMMouse); message wm_LButtonUp;
    procedure WMMouseMove(var M: TWMMouse); message wm_MouseMove;
    procedure WMPaint(var M: TMessage); message WM_Paint;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    { Protected declarations }
    procedure WMNCHitTest(var M: TWMNCHitTest); message wm_NCHitTest;
  public
    { Public declarations }
    BtnRect: TRect;
    procedure CreateWnd; override;
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property BtnCaption: String read fBtnCaption write SetBtnCaption;
    property ButtonWidth: Integer read fButtonWidth write SetButtonWidth;
    property OnButtonClick: TNotifyEvent read fOnButtonClick write fOnButtonClick;
  end;

procedure Register;

implementation

constructor TBtnEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fButtonWidth:=19;
  fInButton:=false;
  fPressed:=false;
  fBtnCaption:='...';
end;

procedure TBtnEdit.CreateWnd;
begin
  inherited CreateWnd;
  Perform(EM_SETMARGINS,EC_RIGHTMARGIN,(fButtonWidth) shl 16);
  BtnRect:=ClientRect;
  BtnRect.Left:=BtnRect.Right-fButtonWidth;
end;

procedure TBtnEdit.WMLButtonDown(var M: TWMMouse);
begin
  if M.XPos>=(ClientWidth-fButtonWidth) then
  begin
    fPressed:=true;
    Invalidate;
    SetCapture(Handle);
  end
  else inherited;
end;

procedure TBtnEdit.WMLButtonUp(var M: TWMMouse);
var PT: TPoint;
begin
  if fPressed then
  begin
    fPressed:=false;
    InvalidateRect(Handle,@BtnRect,True);
    GetCursorPos(PT);
    If PtInRect(BtnRect,ScreenToClient(PT)) and Assigned(fOnButtonClick) then
      fOnButtonClick(Self);
    ReleaseCapture;
  end
  else inherited;
end;

procedure TBtnEdit.WMMouseMove(var M: TWMMouse);
begin
  inherited;
  if M.XPos>=(ClientWidth-fButtonWidth) then
  begin
    cursor:=crArrow;
    if not fInButton and PtInRect(BtnRect,Point(M.XPos,M.YPos)) then
    begin
      fInButton:=true;
      if fPressed then
      InvalidateRect(Handle,@BtnRect,True);
    end;
  end
  else
  begin
    cursor:=crDefault;
    if fInButton then
    begin
      fInButton:=false;
      if fPressed then
      InvalidateRect(Handle,@BtnRect,True);
    end;
  end;
end;

procedure TBtnEdit.WMPaint(var M: TMessage);
var DC: HDC;
    Fnt: HFont;
begin
  inherited;
  DC:=GetDC(Handle);
  Fnt:=Font.Handle;
  SelectObject(DC,Fnt);
  SetBkMode(DC,TRANSPARENT);
  BtnRect:=ClientRect;
  BtnRect.Left:=BtnRect.Right-fButtonWidth;
  if fPressed and fInButton then
  begin
    DrawFrameControl(DC,BtnRect,DFC_BUTTON,DFCS_BUTTONPUSH+DFCS_PUSHED);
    DrawText(DC,PChar(fBtnCaption),Length(fBtnCaption),BtnRect,DT_CENTER or DT_SINGLELINE or DT_VCENTER);
  end
  else
  begin
    DrawFrameControl(DC,BtnRect,DFC_BUTTON,DFCS_BUTTONPUSH);
    DrawText(DC,PChar(fBtnCaption),Length(fBtnCaption),BtnRect,DT_CENTER or DT_SINGLELINE or DT_VCENTER);
  end;
//  DeleteObject(Fnt);
//  ReleaseDC(Handle,DC);
end;

procedure TBtnEdit.WMNCHitTest(var M: TWMNCHitTest);
begin
  If (GetKeyState(VK_MENU) and $80)=$80 then
    M.Result:=HTCAPTION
  else
    M.Result:=HTCLIENT;
end;

procedure TBtnEdit.CMFontChanged(var Message: TMessage);
begin
  inherited;
  BtnRect:=ClientRect;
  BtnRect.Left:=BtnRect.Right-fButtonWidth;
  Invalidate;
end;

procedure TBtnEdit.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if fInButton then
  begin
    fInButton:=false;
    if fPressed then
      InvalidateRect(Handle,@BtnRect,True);
  end;
end;

procedure TBtnEdit.SetButtonWidth(Value: Integer);
begin
  if fButtonWidth<>Value then
  begin
    fButtonWidth:=Value;
    Perform(EM_SETMARGINS,EC_RIGHTMARGIN,(fButtonWidth) shl 16);
    BtnRect.Left:=BtnRect.Right-fButtonWidth;
    Invalidate;
  end;
end;

procedure TBtnEdit.SetBtnCaption(Value: String);
begin
  if fBtnCaption<>Value then
  begin
    fBtnCaption:=Value;
    Invalidate;
  end;
end;

procedure Register;
begin
  RegisterComponents('Campulka', [TBtnEdit]);
end;

end.
